#ifndef parse_date_h_included
#define parse_date_h_included


//
// Copyright 2002, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.
//
// Warning:  not all files in this directory structure are covered by the
// same copyright.  Some of them are part of the GNU source distribution
// and you must obey the GPL copyright for those files.

///@file parse_date.h
///
///   This file defines the interface to the parse_date function.  
///   This  function was written by David Nugent and is released with his own
///   copyright -- although you could easily rewrite it yourself understanding
///   what he does.
//


#include <time.h>

time_t parse_date(char const *string, time_t dt=0);

  //
  ///  The parse_date function was written by David Nugent and he released
  ///  it with his own shareable copyright.  See the lib/parse_date.c file.
  ///  Lowell Boggs modified it in a trivial manner to fix a perceived bug.
  ///  see the source for details.
  ///
  ///  Note also that this is a heavy weight routine. If you need to do a 
  ///  lot of date parsing, you might considering using a routine that does
  ///  not need to accept so many alternative formats -- such as 
  ///  FileTime::construct_from_string().  The parse_date function should
  ///  be used when user input is parsed.  Machine generated date string,
  ///  as from email Date: options are likely to be good candidates for this
  ///  as well due to the diviersity of technique used by mail authoring tools.
  ///
  ///  The function parse_date function reads a string and returns a
  ///  time_t.  There are two interfaces:  one that takes a default date
  ///  and the other that relies only on the current date as the default.
  ///  The default date is only used if the string being parsed does not 
  ///  specify the date but only the time.  The default date would be created 
  ///  by mktime() or would be inquired using time(), etc.
  ///
  ///  The parse date function accepts a variety of strings as input and
  ///  does its best to give you a full and complete interpretation.  Here
  ///  are some examples:
  ///
  ///  @code
  ///    24/12/02
  ///    02/02/99
  ///    15/07/2002
  ///    02/01/02 10:45
  ///    01/03/02 10:45:01
  ///    Mon, Jan 13 1980
  ///    Tues, Dec 7 1941 10:45 -05:00
  ///    Feb 14 1918 10:45
  ///    Feb 14 2002 10:45:03
  ///    Sat, October 13, 2003 09:15:59 -05:00
  ///  @endcode
  ///
  ///  Note that you must put the time after the year if you decide to
  ///  specify the time. 
  ///
  ///  There is no error checking done.  If you use parse_date on an invalid
  ///  date string you will just get wierd results.


#endif
