//
// Copyright 1998, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.
#ifndef _MSC_VER
// on unix only

#include <stdio.h>
#include <stdlib.h>
#include <portable_io.h>
#include <errno.h>
#include <cxxtls/controlledprocess.h>


#define READ_HANDLE  0
#define WRITE_HANDLE 1

namespace cxxtls
{
    ControlledProcess *ControlledProcess::start(std::string command_line,
                                                int inputBuf,
                                                int outputBuf
                                               )
    {
      ControlledProcess *rvPtr = new ControlledProcess();

      ControlledProcess &rv(*rvPtr);
    
      rv.status_             = general_failure;
      rv.command_invocation_ = command_line;
    
      int read_from[2];
      int write_to[2];
    
      if(pipe(read_from))
      {
        //no files yet open, nothing to close

        rv.status_ = can_not_create_pipes;
        return rvPtr;
      }
    
      if(pipe(write_to))
      {
        // the read files opened ok but not the write files

        close(read_from[0]);
        close(read_from[1]);
    
        rv.status_ = can_not_create_pipes;
        return rvPtr;
    
      }
    
      rv.process_stdin_fh_ = write_to[WRITE_HANDLE];
      rv.process_stdout_fh_= read_from[READ_HANDLE];
    
      fflush(stdout);
    
      rv.process_ = fork();
    
      if(rv.process_ == 0)
      {
        // in the child
    
        close(write_to[WRITE_HANDLE]);    // close unneeded handles to prevent os wierdnesses
        close(read_from[READ_HANDLE]);
    
        close(0);  // close stdin
    
        if( 0 != dup(write_to[READ_HANDLE]) )
        {
          // fprintf(stderr, "in child process:  dup() didn't give file descriptor 0\n");

          exit(1);
        }
    
        close(write_to[READ_HANDLE]);  // using stdin from now on
    
        close(1);  // close stdout
    
        if(1 != dup(read_from[WRITE_HANDLE]))
        {
          // fprintf(stderr, "in child process:  dup() didn't give file descriptor 1\n");

          exit(1);
        }
    
        close(read_from[WRITE_HANDLE]);  // using stdout from now on
    
        { // convert command string into a vector of parameters and execute it
    
          execlp("sh", "sh", "-c", rv.command_invocation_.c_str(), (char*)(0));
        }

        // should not get here, but if we do, die...

        // fprintf(stderr, "exec failed on '%s':  errno = %d\n",
        //                 rv.command_invocation_.data(),
        //                 errno
        //        );
    
        exit(1);
    
      }
    
      // in the parent
    
      close(read_from[WRITE_HANDLE]);
      close(write_to[READ_HANDLE]);
    
      rv.process_stdin_fh_ = write_to[WRITE_HANDLE];
      rv.process_stdout_fh_= read_from[READ_HANDLE];
    
      rv.process_stdin_ = fdopen(rv.process_stdin_fh_,"w");
      rv.process_stdout_= fdopen(rv.process_stdout_fh_,"r");

      // file buffering modes
      //
      //   _IONBF unbuffered
      //
      //   _IOLBF line buffered
      //
      //   _IOFBF fully buffered


      int inputBufferingMode  =   _IOFBF;
      int outputBufferingMode =   _IOFBF;

      if(inputBuf == LINE_BUFFERED)
          inputBufferingMode  =   _IOLBF;
      else
      if(inputBuf == CHARACTER_BUFFERED)
          inputBufferingMode  =   _IONBF;

      if(outputBuf == LINE_BUFFERED)
          outputBufferingMode  =   _IOLBF;
      else
      if(outputBuf == CHARACTER_BUFFERED)
          outputBufferingMode  =   _IONBF;

      // in the following two calls, the buffer ptr is NULL, therefore the size field is
      // ignored -- zeros are passed for both.

      setvbuf(rv.process_stdout_, 0, outputBufferingMode, 0);
      setvbuf(rv.process_stdin_,  0, inputBufferingMode, 0);

      rv.status_ = 0;
    
      return rvPtr;
    }
    

} // cxxtls
#else
int  please_ignore_this_controlledprocess_variable_declaration;
#endif
